@@ -1,27 +1,27 @@
 // Copyright (C) 1999-2005 Open Source Telecom Corporation.
-//  
+//
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation; either version 2 of the License, or
 // (at your option) any later version.
-// 
+//
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software 
+// along with this program; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
-// 
+//
 // As a special exception, you may use this file as part of a free software
 // library without restriction.  Specifically, if other files instantiate
 // templates or use macros or inline functions from this file, or you compile
 // this file and link it with other files to produce an executable, this
 // file does not by itself cause the resulting executable to be covered by
-// the GNU General Public License.  This exception does not however    
+// the GNU General Public License.  This exception does not however
 // invalidate any other reasons why the executable file might be covered by
-// the GNU General Public License.    
+// the GNU General Public License.
 //
 // This exception applies only to the code released under the name GNU
 // ccRTP.  If you copy code from other releases into a copy of GNU
@@ -43,6 +43,7 @@
 
 #include "private.h"
 #include <ccrtp/rtppkt.h>
+#include <ccrtp/CryptoContext.h>
 
 #ifdef	CCXX_NAMESPACES
 namespace ost {
@@ -87,7 +88,7 @@ uint32 StaticPayloadFormat::staticAudioTypesRates[] = {
 };
 
 StaticPayloadFormat::StaticPayloadFormat(StaticPayloadType type)
-{ 
+{
 	setPayloadType( (type <= lastStaticPayloadType)? type : 0);
 	if ( type <= sptG729 ) {
 		// audio static type
@@ -95,46 +96,46 @@ StaticPayloadFormat::StaticPayloadFormat(StaticPayloadType type)
 	} else {
 		// video static type
 		setRTPClockRate(90000);
-	} 
+	}
 }
 
 DynamicPayloadFormat::DynamicPayloadFormat(PayloadType type, uint32 rate)
-{ 
+{
 	PayloadFormat::setPayloadType(type);
 	setRTPClockRate(rate);
 }
 
 // constructor commonly used for incoming packets
-RTPPacket::RTPPacket(const unsigned char* const block, size_t len, 
+RTPPacket::RTPPacket(const unsigned char* const block, size_t len,
 		     bool duplicate):
 	total((uint32)len),
 	duplicated(duplicate)
 {
 
-	const RTPFixedHeader* const header = 
+	const RTPFixedHeader* const header =
 		reinterpret_cast<const RTPFixedHeader*>(block);
 	hdrSize = sizeof(RTPFixedHeader) + (header->cc << 2);
 	if ( header->extension ){
 		RTPHeaderExt *ext = (RTPHeaderExt *)(block + hdrSize);
-		hdrSize += sizeof(uint32) + ntohl(ext->length);
+                hdrSize += sizeof(uint32) + (ntohs(ext->length) * 4);
 	}
-	if ( header->padding ) 
+	if ( header->padding )
 		len -= block[len - 1];
 	payloadSize = (uint32)(len - hdrSize);
-	
-	if ( duplicate ) {
+
+        if ( duplicate ) {
 		buffer = new unsigned char[len];
 		setbuffer(block,len,0);
 	} else {
 		buffer = const_cast<unsigned char*>(block);
 	}
 }
-	
+
 // constructor commonly used for outgoing packets
-RTPPacket::RTPPacket(size_t hdrlen, size_t plen, uint8 paddinglen) :
-	buffer(NULL),
-	hdrSize((uint32)hdrlen),
+RTPPacket::RTPPacket(size_t hdrlen, size_t plen, uint8 paddinglen, CryptoContext* pcc ) :
 	payloadSize((uint32)plen),
+        buffer(NULL),
+        hdrSize((uint32)hdrlen),
 	duplicated(false)
 {
 	total = (uint32)(hdrlen + payloadSize);
@@ -144,8 +145,19 @@ RTPPacket::RTPPacket(size_t hdrlen, size_t plen, uint8 paddinglen) :
 		padding = paddinglen - (total % paddinglen);
 		total += padding;
 	}
-	// now we know the actual total length of the packet
-	buffer = new unsigned char[total];
+        srtpLength = 0;
+        srtpDataOffset = 0;
+        if (pcc != NULL) {
+                // compute additional memory for SRTP data
+                srtpLength = pcc->getTagLength() + pcc->getMkiLength();
+                srtpDataOffset = total; // SRTP data go behind header plus payload plus padding
+        }
+
+	// now we know the actual total length of the packet, get  some memory
+        // but take SRTP data into account. Don't change total because some RTP
+        // functions rely on the fact that total is the overall size (without
+        // the SRTP data)
+	buffer = new unsigned char[total + srtpLength];
 	*(reinterpret_cast<uint32*>(getHeader())) = 0;
 	getHeader()->version = CCRTP_VERSION;
 	if ( 0 != padding ) {
@@ -157,7 +169,7 @@ RTPPacket::RTPPacket(size_t hdrlen, size_t plen, uint8 paddinglen) :
 	}
 }
 
-void 
+void
 RTPPacket::endPacket()
 {
 #ifdef	CCXX_EXCEPTIONS
@@ -170,11 +182,12 @@ RTPPacket::endPacket()
 }
 
 OutgoingRTPPkt::OutgoingRTPPkt(
-	const uint32* const csrcs, uint16 numcsrc, 
+	const uint32* const csrcs, uint16 numcsrc,
         const unsigned char* const hdrext, uint32 hdrextlen,
-	const unsigned char* const data, size_t datalen, uint8 paddinglen= 0) :
-	RTPPacket((getSizeOfFixedHeader() + sizeof(uint32) * numcsrc 
-		  + hdrextlen),datalen,paddinglen)
+	const unsigned char* const data, size_t datalen,
+        uint8 paddinglen, CryptoContext* pcc) :
+	RTPPacket((getSizeOfFixedHeader() + sizeof(uint32) * numcsrc
+		  + hdrextlen),datalen,paddinglen, pcc)
 {
 	uint32 pointer = (uint32)getSizeOfFixedHeader();
 	// add CSCR identifiers (putting them in network order).
@@ -187,14 +200,16 @@ OutgoingRTPPkt::OutgoingRTPPkt(
 	pointer += hdrextlen;
 
 	// add data.
-	setbuffer(data,datalen,pointer);
+        setbuffer(data,datalen,pointer);
+        zrtpChecksumLength = 0;
 }
 
 OutgoingRTPPkt::OutgoingRTPPkt(
-	const uint32* const csrcs, uint16 numcsrc, 
-	const unsigned char* data, size_t datalen, uint8 paddinglen = 0) :
+	const uint32* const csrcs, uint16 numcsrc,
+	const unsigned char* data, size_t datalen,
+        uint8 paddinglen, CryptoContext* pcc) :
 	RTPPacket((getSizeOfFixedHeader() + sizeof(uint32) *numcsrc),datalen,
-		  paddinglen)
+		  paddinglen, pcc)
 {
 	uint32 pointer = (uint32)getSizeOfFixedHeader();
 	// add CSCR identifiers (putting them in network order).
@@ -207,11 +222,12 @@ OutgoingRTPPkt::OutgoingRTPPkt(
 
 	// add data.
 	setbuffer(data,datalen,pointer);
+        zrtpChecksumLength = 0;
 }
 
-OutgoingRTPPkt::OutgoingRTPPkt(const unsigned char* data, size_t datalen, 
-			       uint8 paddinglen = 0) :
-	RTPPacket(getSizeOfFixedHeader(),datalen,paddinglen)
+OutgoingRTPPkt::OutgoingRTPPkt(const unsigned char* data, size_t datalen,
+			       uint8 paddinglen, CryptoContext* pcc) :
+	RTPPacket(getSizeOfFixedHeader(),datalen,paddinglen, pcc)
 {
 	// not needed, as the RTPPacket constructor sets by default
 	// the whole fixed header to 0.
@@ -219,6 +235,7 @@ OutgoingRTPPkt::OutgoingRTPPkt(const unsigned char* data, size_t datalen,
 	//getHeader()->extension = 0;
 
 	setbuffer(data,datalen,getSizeOfFixedHeader());
+        zrtpChecksumLength = 0;
 }
 
 void
@@ -229,7 +246,60 @@ OutgoingRTPPkt::setCSRCArray(const uint32* const csrcs, uint16 numcsrc)
 	for ( int i = 0; i < numcsrc; i++ )
 		csrc[i] = htonl(csrc[i]);
 	getHeader()->cc = numcsrc;
-}	
+}
+
+void
+OutgoingRTPPkt::protect(uint32 ssrc, CryptoContext* pcc)
+{
+        /* Encrypt the packet */
+        uint64 index = ((uint64)pcc->getRoc() << 16) | (uint64)getSeqNum();
+
+        // if it's a ZRTP packet adjust data to accomodate ZRTP checksum
+        srtpDataOffset -= zrtpChecksumLength;
+        total -= zrtpChecksumLength;
+        payloadSize -= zrtpChecksumLength;
+
+        pcc->srtpEncrypt(this, index, ssrc);
+
+        // NO MKI support yet - here we assume MKI is zero. To build in MKI
+        // take MKI length into account when storing the authentication tag.
+
+        /* Compute MAC */
+        pcc->srtpAuthenticate(this, pcc->getRoc(),
+                                   const_cast<uint8*>(getRawPacket()+srtpDataOffset) );
+        total += zrtpChecksumLength;
+
+        /* Update the ROC if necessary */
+        if (getSeqNum() == 0xFFFF ) {
+                pcc->setRoc(pcc->getRoc() + 1);
+        }
+}
+
+#define CKSUM_CARRY(x) (x = (x >> 16) + (x & 0xffff), (~(x + (x >> 16)) & 0xffff))
+void
+OutgoingRTPPkt::computeZrtpChecksum()
+{
+    uint8* cdata = const_cast<uint8*>(getRawPacket());
+    uint16* data =(uint16*)cdata;
+
+    int32 length = total + srtpLength - zrtpChecksumLength;
+
+    uint32_t sum = 0;
+    uint16_t ans = 0;
+
+    while (length > 1) {
+        sum += *data++;
+        length -= 2;
+    }
+    if (length == 1) {
+        *(uint8_t *)(&ans) = *(uint8_t*)data;
+        sum += ans;
+    }
+    uint16 ret = CKSUM_CARRY(sum);
+
+    uint8* tmp = const_cast<uint8*>(getRawPacket());
+    memcpy(tmp+total+srtpLength-zrtpChecksumLength, &ret, 2);
+}
 
 // These masks are valid regardless of endianness.
 const uint16 IncomingRTPPkt::RTP_INVALID_PT_MASK = (0x7e);
@@ -238,16 +308,18 @@ const uint16 IncomingRTPPkt::RTP_INVALID_PT_VALUE = (0x48);
 IncomingRTPPkt::IncomingRTPPkt(const unsigned char* const block, size_t len) :
 	RTPPacket(block,len)
 {
-	// first, perform validity check: 
+	// first, perform validity check:
 	// 1) check protocol version
 	// 2) it is not an SR nor an RR
 	// 3) consistent length field value (taking CC value and P and
 	//    X bits into account)
-	if ( getProtocolVersion() != CCRTP_VERSION 
+	if ( getProtocolVersion() != CCRTP_VERSION
 	     ||
-	     (getPayloadType() & RTP_INVALID_PT_MASK) == RTP_INVALID_PT_VALUE
+             (getPayloadType() & RTP_INVALID_PT_MASK) == RTP_INVALID_PT_VALUE) {
+            /*
 	     ||
 	     getPayloadSize() <= 0 ) {
+            */
 		headerValid = false;
 		return;
 	}
@@ -257,6 +329,77 @@ IncomingRTPPkt::IncomingRTPPkt(const unsigned char* const block, size_t len) :
 	cachedSSRC = ntohl(getHeader()->sources[0]);
 }
 
+bool
+IncomingRTPPkt::checkZrtpChecksum(bool check)
+{
+    // TODO do a real check, for now adjust length only
+    total -= 2;
+    if (payloadSize >= 2) {
+        payloadSize -= 2;
+    }
+    return true;
+}
+
+int32
+IncomingRTPPkt::unprotect(CryptoContext* pcc)
+{
+        if (pcc == NULL) {
+                return true;
+        }
+
+        /*
+         * This is the setting of the packet data when we come to this
+         * point:
+         *
+         * total:       complete length of received data
+         * buffer:      points to data as received from network
+         * hdrSize:     length of header including header extension
+         * payloadSize: length of data excluding hdrSize and padding
+         *
+         * Because this is an SRTP packet we need to adjust some values here.
+         * The SRTP MKI and authentication data is always at the end of a
+         * packet. Thus compute the position of this data.
+         */
+
+        uint32 srtpDataIndex = total - (pcc->getTagLength() + pcc->getMkiLength());
+
+        // now adjust total because some RTP functions rely on the fact that
+        // total is the full length of data without SRTP data.
+        total -= pcc->getTagLength() + pcc->getMkiLength();
+
+        // recompute payloadSize by subtracting SRTP data
+        payloadSize -= pcc->getTagLength() + pcc->getMkiLength();
+
+	// unused??
+        // const uint8* mki = getRawPacket() + srtpDataIndex;
+        const uint8* tag = getRawPacket() + srtpDataIndex + pcc->getMkiLength();
+
+        /* Replay control */
+        if (!pcc->checkReplay(cachedSeqNum)) {
+                return -2;
+        }
+        /* Guess the index */
+        uint64 guessedIndex = pcc->guessIndex(cachedSeqNum);
+
+        uint32 guessedRoc = guessedIndex >> 16;
+        uint8* mac = new uint8[pcc->getTagLength()];
+
+        pcc->srtpAuthenticate(this, guessedRoc, mac);
+        if (memcmp(tag, mac, pcc->getTagLength()) != 0) {
+            delete[] mac;
+            return -1;
+        }
+        delete[] mac;
+
+        /* Decrypt the content */
+        pcc->srtpEncrypt( this, guessedIndex, cachedSSRC );
+
+        /* Update the Crypto-context */
+        pcc->update(cachedSeqNum);
+
+        return 1;
+}
+
 #ifdef	CCXX_NAMESPACES
 }
 #endif
